//----------------------------------------------------------------------------------------------------------------------
#include "../io/Console.hpp"
//----------------------------------------------------------------------------------------------------------------------
#include "EffectManager.hpp"
#include "Effect.hpp"
//----------------------------------------------------------------------------------------------------------------------
#include <fstream>
//----------------------------------------------------------------------------------------------------------------------
using namespace std;
//----------------------------------------------------------------------------------------------------------------------
namespace bleifrei {
//----------------------------------------------------------------------------------------------------------------------
gfx::EffectManager Singleton<gfx::EffectManager>::instance;
//----------------------------------------------------------------------------------------------------------------------
	namespace gfx {
//----------------------------------------------------------------------------------------------------------------------

EffectManager::EffectManager(void)
{
}

//----------------------------------------------------------------------------------------------------------------------

EffectManager::~EffectManager(void)
{
	destroy();
}

//----------------------------------------------------------------------------------------------------------------------

bool
EffectManager::init(const string &path)
{
	m_cgContext = cgCreateContext();
	if (!m_cgContext)
	{
		io::Console::getInstance().printf(1, 0, 0, "Failed to initialize EffectManager!");
		return false;
	}
	cgGLRegisterStates(m_cgContext);

	this->m_effectPath = path;
	
	size_t pathLength = path.length();
	if (pathLength)
	{
		char last = path.at(pathLength - 1);

		if (last != '\\' && last != '/')
		{
			m_effectPath += '/';
		}
	}

	return true;
}

//----------------------------------------------------------------------------------------------------------------------

void
EffectManager::destroy(void)
{
	map<string, Effect *>::iterator it;
	for (it = m_effects.begin(); it != m_effects.end(); it++)
	{
		if (it->second) it->second->release();
	}
	m_effects.clear();

	cgDestroyContext(m_cgContext);
}

//----------------------------------------------------------------------------------------------------------------------

Effect *
EffectManager::load(const string &filename)
{
	Effect *effect;

	map<string, Effect *>::iterator it = m_effects.find(filename);

	if (it != m_effects.end())
	{
		effect = it->second;
	}
	else
	{
		effect = new Effect();
		if (!effect->load(m_effectPath + filename))
		{
			effect->release();
			return 0;
		}

		m_effects.insert(map<string, Effect *>::value_type(filename, effect));
	}

	return effect;
}

//----------------------------------------------------------------------------------------------------------------------

Effect *
EffectManager::reload(const string &filename)
{
	map<string, Effect *>::iterator it = m_effects.find(filename);
	if (it != m_effects.end())
	{
		safe_release(it->second);
		m_effects.erase(it);
	}
	return load(filename);
}

//----------------------------------------------------------------------------------------------------------------------
	}
}
//----------------------------------------------------------------------------------------------------------------------
