
#ifndef _include_bleifrei_platform_Window_hpp_
#define _include_bleifrei_platform_Window_hpp_ 


// includes
#include <string>

#include "../gfx/RenderTarget.hpp"
#include "../Singleton.hpp"

namespace bleifrei {
	namespace platform {

		class window_creation_failed {};

		class WindowEventListener {
		public:
			virtual void onClose(void);
			virtual void onResize(int width, int height);
		};

		class Window : public gfx::RenderTarget, public Singleton<Window> {
			friend class Singleton<Window>;
		public:
			bool init(const std::string &title, int width, int height, int bpp, bool fullscreen);
			void destroy(void);

			void setEventListener(WindowEventListener *eventListener);

			bool isFullscreen(void) const										{ return fullscreen;					}
			void toLocal(int &x, int &y);

			void closeEvent(void);
			void resizeEvent(int width, int height);

		private:
			Window(void) : eventListener(0), windowHandle(NULL) {}
			~Window(void) {}

			bool fullscreen;
			std::string title;

			WindowEventListener *eventListener;

		// Windows specific
		private:
			bool windows_fullscreen(void);
			void windows_pixelFormat(void);

			HWND windowHandle;
			HWND m_startMenu;
		};

		inline
		void
		Window::closeEvent(void)
		{
			if (eventListener) eventListener->onClose();
		}

		inline
		void
		Window::resizeEvent(int width, int height)
		{
			this->width = width;
			this->height = height;
			if (eventListener) eventListener->onResize(width, height);
		}

	}
}


#endif //include_bleifrei_platform_Window_hpp_
