/** 
 * bleifrei utility library
 *
 * Copyright (C) 2003 Ulf Borgenstam, Jonas Svensson.
 *
 * Do not distribute. If you know someone interested in obtaining these sources
 * please have them contact us.
 */

#ifndef _include_bleifrei_Mesh_hpp_
#define _include_bleifrei_Mesh_hpp_


// includes
#include <iostream>
#include <string>
#include <vector>

#include "../math/Matrix.hpp"
#include "../platform/types.hpp"
#include "../Base.hpp"

namespace bleifrei {
	namespace scene {

		class Mesh : public Base {
		public:
			Mesh(void);
			virtual ~Mesh(void);

			void setName(const std::string &name)							{ this->name = name;				}
			const std::string &getName(void)								{ return name;						}

			void append(Mesh *mesh);

			unsigned int addVertex(const math::Vector3f &v);
			unsigned int addTriangle(const math::Vector3ui &triangleIndices, unsigned int materialIndex = 0);

			void setVertex(unsigned int index, const math::Vector3f &vertex);
			void setNormal(unsigned int index, const math::Vector3f &normal);
			void setTexCoord(unsigned int index, const math::Vector2f &texCoord);

			std::vector<math::Vector3f>	&getVertices(void)					{ return vertices;					}
			std::vector<math::Vector3f>	&getNormals(void)					{ return normals;					}
			std::vector<math::Vector2f>	&getTexCoords(void)					{ return texcoords;					}
			std::vector<math::Vector3ui> &getTriangles(void)				{ return triangles;					}
			std::vector<unsigned int> &getMaterialIndices(void)				{ return materials;					}

			void read(std::istream &is);
			void write(std::ostream &os);

		private:
			std::vector<math::Vector3f> vertices;
			std::vector<math::Vector3f> normals;
			std::vector<math::Vector2f> texcoords;

			std::vector<math::Vector3ui> triangles;
			std::vector<unsigned int> materials;

			std::string name;
		};

	}
}

#endif //_include_bleifrei_Mesh_hpp_
